CON
  _CLKMODE      = XTAL1 + PLL16X                        
  _XINFREQ      = 5_000_000

OBJ
   term:    "FullDuplexSerial_NP"
   desc:    "COGdesc"

VAR
  long buffer[ 128 ]
  long ld_buffer[ 512 ]

  long xPos,yPos,zPos, dx, dy, dz, dxrem, dyrem, dzrem, xTarg, yTarg, zTarg
  long xsum,ysum,zsum,xinc,yinc,zinc,steps

  long tmp, i, j, k, offset, dmask,start,end

  long stack[60]
  
PUB stepperTest
  ' if RAM descriptor table should be used, it has to be initialized first by
  ' setting the number of descriptor entries
  desc.initRAMDesc( 15 )
  ' this initializes the parameter-manager which can be asked for upper memory
  desc.initPARmng

  ' start the code that knows how to read the driver table
  ' and give it some space ( 512 bytes for directory + 512 longs for driver)
  ' after loading all needed drivers the buffers can be used for other things 
  i:=\desc.initDesc( @buffer, desc#DEV_EEPROM )
  ' if it is no EEPROM device, this has failed and a retry for FRAM makes sense

  ' load the FullDuplexSerial in version 1.1
  tmp := desc.loadDriver( @fdserial , $0110 )
  ' when you are sure that the driver exists it can be started right away, otherwise
  ' you should rather check the result
  if tmp<0
    ' not even the serial driver is there, so stop here
    return

  ' get some memory from upper RAM
  tmp := desc.parReserve( desc.getPSize )
  
  ' run the NP full duplex
  ' first get the address of the communication buffer  ' tmp :=
  'tmp := term.start( term#BOOT_RX, term#BOOT_TX, 0, 57600 )
  term.start( term#BOOT_RX, term#BOOT_TX, 0, 115200, tmp )
  ' and then start the code loaded by loadDriver
  i := desc.startDriver(@ld_buffer, tmp)

  ' wait until any key has been pushed or a minute has passed

  tmp:=-1
  j:=0
  repeat while (tmp==-1) and (j<60)
    term.str( string( "StepperTest",$0d,"== please press key to start ==",$0d ) )
    tmp:=term.rxtime( 1000 )
    j++

  cls

  dira[ 23 ]:=1
  
  cognew( xAxis, @stack )
  cognew( yAxis, @stack[20] )
  cognew( zAxis, @stack[40] )
  waitcnt( clkfreq+cnt )

  xPos:=0
  yPos:=0
  zPos:=0

  goTo( 18, 7, 0 )
  goTo( 50, 50, 50 )
  'goTo( 0,0,0 )

PUB xAxis
  dira[ 16 ]:=1
  repeat
    waitpeq( |<23 , |<23, 0 )
    'start:=cnt
    xsum+=||dxrem
    if xsum=>steps
      xPos+=xinc
      xsum-=steps
      outa[ 16 ]:=1
    else
      outa[ 16 ]:=0
    'end:=cnt
    waitpne( |<23 , |<23, 0 )
    
PUB yAxis
  dira[ 17 ]:=1
  repeat
    waitpeq( |<23 , |<23, 0 )
    start:=cnt
    ysum+=||dyrem
    if ysum=>steps
      yPos+=yinc
      ysum-=steps
      outa[ 17 ]:=1
    else
      outa[ 17 ]:=0
    end:=cnt
    waitpne( |<23 , |<23, 0 )

PUB zAxis
  dira[ 18 ]:=1
  repeat
    waitpeq( |<23 , |<23, 0 )
    zsum+=||dzrem
    if zsum=>steps
      zPos+=zinc
      zsum-=steps
      outa[ 18 ]:=1
    else
      outa[ 18 ]:=0
    waitpne( |<23 , |<23, 0 )

PUB goTo( x,y,z ) 
  dx:=x-xPos
  if x>xPos
    xinc:=1
  else
    xinc:=-1

  dy:=y-yPos
  if y>yPos
    yinc:=1
  else
    yinc:=-1

  dz:=z-zPos
  if z>zPos
    zinc:=1
  else
    zinc:=-1

  xTarg:=x
  yTarg:=y
  zTarg:=z
  
  if( (||dx)>(||dy)) and ((||dx)>(||dz) )
    steps:=||dx
  else
    if( (||dy)>(||dz) )
      steps:=||dy
    else
      steps:=||dz

  stepper

PUB stepper | stepnumber
  dxrem:=dx//(steps+1)
  dyrem:=dy//(steps+1)
  dzrem:=dz//(steps+1)

  xsum:=steps/2
  ysum:=steps/2
  zsum:=steps/2
  stepnumber:=0
  
  term.str( string( "start ") )
  term.dec( xPos )
  term.tx( " " )  
  term.dec( yPos )
  term.tx( " " )  
  term.dec( zPos )
  nl
  repeat steps
    outa[ 23 ]:=1
    stepnumber++
    waitcnt( clkfreq/100+cnt )
    outa[ 23 ]:=0
    term.dec( stepnumber )
    term.tx( " " )
    waitcnt( clkfreq/100+cnt )
    term.dec( xPos )
    term.tx( " " )  
    term.dec( yPos )
    term.tx( " " )  
    term.dec( zPos )
    nl

PUB cls
  term.tx( 0 )
  term.tx( 1 )

PUB nl
  term.tx(13)

PUB ok
  term.str( string( "ok",10, 13 ) )
      
DAT
fdserial      byte      "FDSerial",0